@@ -63,7 +63,7 @@ DEFAULT_CONFIG = immutabledict(
         "artifact_dir": "...",
         "task_log_dir": "...",  # set this to ARTIFACT_DIR/public/logs
         "artifact_upload_timeout": 60 * 20,
-        "aiohttp_max_connections": 15,
+        "max_concurrent_downloads": 5,
         # chain of trust settings
         "sign_chain_of_trust": True,
         "verify_chain_of_trust": False,  # TODO True
@@ -80,9 +80,7 @@ DEFAULT_CONFIG = immutabledict(
         "ed25519_private_key_path": "...",
         "ed25519_public_keys": immutabledict(
             {
-                # the key beginning in J+PAK... is the longstanding docker-worker CoT key, to be phased out
-                # the key beginning in tk/Sj... is a new key added to differentiate GCP workers
-                "docker-worker": tuple(["J+PAKmq3jkS2uCpBk5WU2ycrnTFPwZujJT4OHAxm38I=", "tk/SjxY3mREARba6ODw7qReUoVWj0RgEIxBURkwcM4I="]),
+                "docker-worker": tuple(["tk/SjxY3mREARba6ODw7qReUoVWj0RgEIxBURkwcM4I="]),
                 "generic-worker": tuple(["6UPrVTyw0EPQV7bCEMXo+5jNR4clbK55JWG74bBJHZQ="]),
                 "scriptworker": tuple(["DaEKQ79ZC/X+7O8zwm8iyhwTlgyjRSi/TDd63fh2JG0="]),
             }
@@ -111,48 +109,57 @@ DEFAULT_CONFIG = immutabledict(
         "scriptworker_provisioners": ("scriptworker-prov-v1", "scriptworker-k8s"),
         # valid hash algorithms for chain of trust artifacts
         "valid_hash_algorithms": ("sha256", "sha512"),
+        "cot_product_type": immutabledict(
+            {
+                "by-cot-product": immutabledict(
+                    {
+                        "firefox": "hg",
+                        "thunderbird": "hg",
+                        "mobile": "github",
+                        "mpd001": "github",
+                        "app-services": "github",
+                        "glean": "github",
+                        "xpi": "github",
+                        "adhoc": "github",
+                        "scriptworker": "github",
+                    },
+                ),
+            },
+        ),
         # decision task cot
         "valid_decision_worker_pools": immutabledict(
             {
                 "by-cot-product": immutabledict(
                     {
                         "firefox": (
-                            "aws-provisioner-v1/gecko-1-decision",
-                            "aws-provisioner-v1/gecko-2-decision",
-                            "aws-provisioner-v1/gecko-3-decision",
                             "gecko-1/decision",
                             "gecko-2/decision",
                             "gecko-3/decision",
                         ),
                         "thunderbird": (
-                            "aws-provisioner-v1/gecko-1-decision",
-                            "aws-provisioner-v1/gecko-2-decision",
-                            "aws-provisioner-v1/gecko-3-decision",
                             "comm-1/decision",
                             "comm-2/decision",
                             "comm-3/decision",
                         ),
                         "mobile": (
-                            # gecko-focus was for mozilla-mobile releases (bug 1455290) for more details.
-                            # TODO: Remove it once not used anymore
-                            "aws-provisioner-v1/gecko-focus",
-                            "aws-provisioner-v1/mobile-1-decision",
-                            # We haven't had the need for mobile-2-decision yet
-                            # https://bugzilla.mozilla.org/show_bug.cgi?id=1512631#c6
-                            "aws-provisioner-v1/mobile-3-decision",
                             "mobile-1/decision",
                             "mobile-3/decision",
                         ),
-                        "mpd001": ("aws-provisioner-v1/mpd001-1-decision", "aws-provisioner-v1/mpd001-3-decision", "mpd001-1/decision", "mpd001-3/decision"),
-                        "application-services": (
-                            "aws-provisioner-v1/app-services-1-decision",
-                            "aws-provisioner-v1/app-services-3-decision",
+                        "mpd001": ("mpd001-1/decision", "mpd001-3/decision"),
+                        "app-services": (
                             "app-services-1/decision",
                             "app-services-3/decision",
                         ),
+                        "glean": (
+                            "glean-1/decision",
+                            "glean-3/decision",
+                        ),
                         "xpi": ("xpi-1/decision", "xpi-3/decision"),
-                        # TODO: Add adhoc-3 whenever we're ready to go to prod
-                        "adhoc": ("adhoc-1/decision",),
+                        "adhoc": ("adhoc-1/decision", "adhoc-3/decision"),
+                        "scriptworker": (
+                            "scriptworker-1/decision",
+                            "scriptworker-3/decision",
+                        ),
                     }
                 )
             }
@@ -163,37 +170,34 @@ DEFAULT_CONFIG = immutabledict(
                 "by-cot-product": immutabledict(
                     {
                         "firefox": (
-                            "aws-provisioner-v1/gecko-1-images",
-                            "aws-provisioner-v1/gecko-2-images",
-                            "aws-provisioner-v1/gecko-3-images",
                             "gecko-1/images",
                             "gecko-2/images",
                             "gecko-3/images",
                         ),
                         "thunderbird": (
-                            "aws-provisioner-v1/gecko-1-images",
-                            "aws-provisioner-v1/gecko-2-images",
-                            "aws-provisioner-v1/gecko-3-images",
                             "comm-1/images",
                             "comm-2/images",
                             "comm-3/images",
                         ),
                         "mobile": (
-                            "aws-provisioner-v1/mobile-1-images",  # there is no mobile level 2.
-                            "aws-provisioner-v1/mobile-3-images",
                             "mobile-1/images",
                             "mobile-3/images",
                         ),
-                        "mpd001": ("aws-provisioner-v1/mpd001-1-images", "aws-provisioner-v1/mpd001-3-images", "mpd001-1/images", "mpd001-3/images"),
-                        "application-services": (
-                            "aws-provisioner-v1/app-services-1-images",
-                            "aws-provisioner-v1/app-services-3-images",
+                        "mpd001": ("mpd001-1/images", "mpd001-3/images"),
+                        "app-services": (
                             "app-services-1/images",
                             "app-services-3/images",
                         ),
+                        "glean": (
+                            "glean-1/images",
+                            "glean-3/images",
+                        ),
                         "xpi": ("xpi-1/images", "xpi-3/images"),
-                        # TODO: Add adhoc-3 whenever we're ready to go to prod
-                        "adhoc": ("adhoc-1/images",),
+                        "adhoc": ("adhoc-1/images", "adhoc-3/images"),
+                        "scriptworker": (
+                            "scriptworker-1/images",
+                            "scriptworker-3/images",
+                        ),
                     }
                 )
             }
@@ -212,7 +216,7 @@ DEFAULT_CONFIG = immutabledict(
                                     r"^(?P<path>/mozilla-(central|unified))(/|$)",
                                     r"^(?P<path>/integration/(autoland|fx-team|mozilla-inbound))(/|$)",
                                     r"^(?P<path>/releases/mozilla-(beta|release|esr\d+))(/|$)",
-                                    r"^(?P<path>/projects/(birch|jamun|maple|oak))(/|$)",
+                                    r"^(?P<path>/projects/(maple|oak))(/|$)",
                                 ),
                             }
                         ),
@@ -258,11 +262,14 @@ DEFAULT_CONFIG = immutabledict(
                             }
                         ),
                     ),
-                    "application-services": (
+                    "app-services": (
                         immutabledict(
                             {"schemes": ("https", "ssh"), "netlocs": ("github.com",), "path_regexes": (r"^(?P<path>/mozilla/application-services)(/|.git|$)",)}
                         ),
                     ),
+                    "glean": (
+                        immutabledict({"schemes": ("https", "ssh"), "netlocs": ("github.com",), "path_regexes": (r"^(?P<path>/mozilla/glean)(/|.git|$)",)}),
+                    ),
                     "xpi": (
                         immutabledict(
                             {
@@ -277,8 +284,16 @@ DEFAULT_CONFIG = immutabledict(
                             {
                                 "schemes": ("https", "ssh"),
                                 "netlocs": ("github.com",),
-                                # TODO: Change repo to its final name
-                                "path_regexes": tuple([r"^(?P<path>/mozilla-releng/mvp-adhoc)(/|.git|$)"]),
+                                "path_regexes": tuple([r"^(?P<path>/mozilla-releng/(?:adhoc-signing|adhoc-manifest))(/|.git|$)"]),
+                            }
+                        ),
+                    ),
+                    "scriptworker": (
+                        immutabledict(
+                            {
+                                "schemes": ("https", "ssh"),
+                                "netlocs": ("github.com",),
+                                "path_regexes": tuple([r"^(?P<path>/mozilla-releng/scriptworker(?:|-scripts))(/|.git|$)"]),
                             }
                         ),
                     ),
@@ -301,15 +316,29 @@ DEFAULT_CONFIG = immutabledict(
                         "github-release",
                     ),
                     "mpd001": ("cron", "github-pull-request", "github-push", "github-release"),
-                    "application-services": (
+                    "app-services": (
+                        "action",
+                        "cron",
                         # On staging releases, level 1 docker images may be built in the pull-request graph
                         "github-pull-request",
                         # Similarly, docker images can be built on regular push. This is usually the case
                         # for level 3 images
+                        "github-push",
+                        "github-release",
+                    ),
+                    "glean": (
+                        "action",
+                        "cron",
+                        # On staging releases, level 1 docker images may be built in the pull-request graph
+                        "github-pull-request",
+                        # Similarly, docker images can be built on regular push. This is usually the case
+                        # for level 3 images
+                        "github-push",
                         "github-release",
                     ),
                     "xpi": ("action", "cron", "github-pull-request", "github-push", "github-release"),
                     "adhoc": ("action", "github-pull-request", "github-push"),
+                    "scriptworker": ("action", "cron", "github-pull-request", "github-push", "github-release"),
                 }
             )
         },
@@ -320,9 +349,11 @@ DEFAULT_CONFIG = immutabledict(
                     "thunderbird": "",
                     "mobile": "mozilla-mobile",
                     "mpd001": "mozilla-services",
-                    "application-services": "mozilla",
+                    "app-services": "mozilla",
+                    "glean": "mozilla",
                     "xpi": "mozilla-extensions",
                     "adhoc": "mozilla-releng",
+                    "scriptworker": "mozilla-releng",
                 }
             )
         },
@@ -371,8 +402,10 @@ DEFAULT_CONFIG = immutabledict(
                     ),
                     "mobile": immutabledict(
                         {
+                            "project:mobile:android-components:releng:github:project:android-components": "android-components-repo",
                             "project:mobile:android-components:releng:beetmover:bucket:maven-production": "android-components-repo",
                             "project:mobile:android-components:releng:beetmover:bucket:maven-snapshot-production": "android-components-repo",
+                            "project:mobile:fenix:releng:github:project:fenix": "fenix-repo",
                             "project:mobile:fenix:releng:googleplay:product:fenix": "fenix-repo",
                             "project:mobile:fenix:releng:signing:cert:nightly-signing": "fenix-repo",
                             "project:mobile:fenix:releng:signing:cert:beta-signing": "fenix-repo",
@@ -397,13 +430,18 @@ DEFAULT_CONFIG = immutabledict(
                             "project:mpd001:releng:signing:cert:release-signing": "mpd001-repo",
                         }
                     ),
-                    "application-services": immutabledict(
-                        {"project:mozilla:application-services:releng:beetmover:bucket:maven-production": "application-services-repo"}
-                    ),
+                    "app-services": immutabledict({"project:mozilla:app-services:releng:beetmover:bucket:maven-production": "app-services-repo"}),
+                    "glean": immutabledict({"project:mozilla:glean:releng:beetmover:bucket:maven-production": "glean-repo"}),
                     "xpi": immutabledict(
                         {"project:xpi:signing:cert:release-signing": "xpi-manifest-repo", "project:xpi:ship-it:production": "xpi-manifest-repo"}
                     ),
-                    "adhoc": immutabledict({"project:adhoc:signing:cert:release-signing": "adhoc-signing-repo"}),
+                    "adhoc": immutabledict({"project:adhoc:signing:cert:release-signing": "adhoc-signing-repos"}),
+                    "scriptworker": immutabledict(
+                        {
+                            "project:scriptworker:dockerhub:production": "scriptworker-scripts-repo",
+                            "project:scriptworker:pypi:production": "all-production-repos",
+                        }
+                    ),
                 }
             )
         },
@@ -414,41 +452,36 @@ DEFAULT_CONFIG = immutabledict(
                     "firefox": immutabledict(
                         {
                             # Which repos can perform release actions?
-                            # XXX remove /projects/maple and birch when taskcluster relpro
-                            #     migration is tier1 and landed on mozilla-central
-                            # XXX remove /projects/jamun when we no longer run staging releases
-                            #     from it
+                            # XXX remove /projects/maple when we have a
+                            #     different prod signing testing solution
                             "all-release-branches": (
                                 "/releases/mozilla-beta",
                                 "/releases/mozilla-release",
-                                "/releases/mozilla-esr60",
                                 "/releases/mozilla-esr68",
-                                "/projects/birch",
-                                "/projects/jamun",
+                                "/releases/mozilla-esr78",
                                 "/projects/maple",
                             ),
                             # Limit things like pushapk to just these branches
                             "release": ("/releases/mozilla-release",),
                             "beta": ("/releases/mozilla-beta",),
                             "beta-or-release": ("/releases/mozilla-beta", "/releases/mozilla-release"),
-                            "esr": ("/releases/mozilla-esr60", "/releases/mozilla-esr68"),
+                            "esr": (
+                                "/releases/mozilla-esr68",
+                                "/releases/mozilla-esr78",
+                            ),
                             "esr68": ("/releases/mozilla-esr68",),
                             "nightly": ("/mozilla-central",),
                             # Which repos can do nightly signing?
-                            # XXX remove /projects/maple and birch when taskcluster relpro
-                            #     migration is tier1 and landed on mozilla-central
-                            # XXX remove /projects/jamun when we no longer run staging releases
-                            #     from it
+                            # XXX remove /projects/maple when we have a
+                            #     different prod signing testing solution
                             # XXX remove /projects/oak when we no longer test updates against it
                             "all-nightly-branches": (
                                 "/mozilla-central",
                                 "/releases/mozilla-unified",
                                 "/releases/mozilla-beta",
                                 "/releases/mozilla-release",
-                                "/releases/mozilla-esr60",
                                 "/releases/mozilla-esr68",
-                                "/projects/birch",
-                                "/projects/jamun",
+                                "/releases/mozilla-esr78",
                                 "/projects/oak",
                                 "/projects/maple",
                             ),
@@ -456,18 +489,29 @@ DEFAULT_CONFIG = immutabledict(
                                 "/mozilla-central",
                                 "/releases/mozilla-beta",
                                 "/releases/mozilla-release",
-                                "/releases/mozilla-esr60",
                                 "/releases/mozilla-esr68",
+                                "/releases/mozilla-esr78",
                             ),
-                            "all-staging-branches": ("/projects/birch", "/projects/jamun", "/projects/maple"),
                         }
                     ),
                     "thunderbird": immutabledict(
                         {
-                            "all-release-branches": ("/releases/comm-beta", "/releases/comm-esr60", "/releases/comm-esr68"),
+                            "all-release-branches": (
+                                "/releases/comm-beta",
+                                "/releases/comm-esr68",
+                                "/releases/comm-esr78",
+                            ),
                             "beta": ("/releases/comm-beta",),
-                            "esr": ("/releases/comm-esr60", "/releases/comm-esr68"),
-                            "all-nightly-branches": ("/comm-central", "/releases/comm-beta", "/releases/comm-esr60", "/releases/comm-esr68"),
+                            "esr": (
+                                "/releases/comm-esr68",
+                                "/releases/comm-esr78",
+                            ),
+                            "all-nightly-branches": (
+                                "/comm-central",
+                                "/releases/comm-beta",
+                                "/releases/comm-esr68",
+                                "/releases/comm-esr78",
+                            ),
                             "nightly": ("/comm-central",),
                         }
                     ),
@@ -482,9 +526,19 @@ DEFAULT_CONFIG = immutabledict(
                         }
                     ),
                     "mpd001": immutabledict({"mpd001-repo": ("/mozilla-services/guardian-vpn-windows",)}),
-                    "application-services": immutabledict({"application-services-repo": ("/mozilla/application-services",)}),
+                    "app-services": immutabledict({"app-services-repo": ("/mozilla/application-services",)}),
+                    "glean": immutabledict({"glean-repo": ("/mozilla/glean",)}),
                     "xpi": immutabledict({"xpi-manifest-repo": ("/mozilla-extensions/xpi-manifest",)}),
-                    "adhoc": immutabledict({"adhoc-signing-repo": ("/mozilla-releng/mvp-adhoc",)}),
+                    "adhoc": immutabledict({"adhoc-signing-repos": ("/mozilla-releng/adhoc-signing", "/mozilla-releng/adhoc-manifest")}),
+                    "scriptworker": immutabledict(
+                        {
+                            "scriptworker-scripts-repo": ("/mozilla-releng/scriptworker-scripts",),
+                            "all-production-repos": (
+                                "/mozilla-releng/scriptworker",
+                                "/mozilla-releng/scriptworker-scripts",
+                            ),
+                        }
+                    ),
                 }
             )
         },
@@ -493,11 +547,14 @@ DEFAULT_CONFIG = immutabledict(
                 {
                     "firefox": ("decision", "action", "docker-image"),
                     "thunderbird": ("decision", "action", "docker-image"),
+                    # XXX now that we're on taskgraph, we should limit these.
                     "mobile": "any",  # all allowed
                     "mpd001": "any",  # all allowed
-                    "application-services": "any",  # all allowed
+                    "app-services": "any",  # all allowed
+                    "glean": "any",  # all allowed
                     "xpi": "any",  # all allowed
                     "adhoc": "any",  # all allowed
+                    "scriptworker": ("decision", "action", "docker-image"),
                 }
             )
         },
@@ -508,9 +565,11 @@ DEFAULT_CONFIG = immutabledict(
                     "thunderbird": "COMM",
                     "mobile": "MOBILE",
                     "mpd001": "MPD001",
-                    "application-services": "APPSERVICES",
+                    "app-services": "APPSERVICES",
+                    "glean": "GLEAN",
                     "xpi": "XPI",
                     "adhoc": "ADHOC",
+                    "scriptworker": "SCRIPTWORKER",
                 }
             )
         },
