@@ -11,7 +11,6 @@ import asyncio
 import logging
 import os
 import pprint
-import re
 import sys
 import tempfile
 from copy import deepcopy
@@ -21,6 +20,8 @@ import aiohttp
 import dictdiffer
 import jsone
 from immutabledict import immutabledict
+from taskcluster.aio import Queue
+
 from scriptworker.artifacts import download_artifacts, get_artifact_url, get_optional_artifacts_per_task_id, get_single_upstream_artifact_full_path
 from scriptworker.config import apply_product_config, read_worker_creds
 from scriptworker.constants import DEFAULT_CONFIG
@@ -48,6 +49,7 @@ from scriptworker.task import (
     get_worker_pool_id,
     is_action,
     is_try_or_pull_request,
+    retry_get_task_definition,
 )
 from scriptworker.utils import (
     add_enumerable_item_to_dict,
@@ -67,8 +69,6 @@ from scriptworker.utils import (
     write_to_file,
 )
 from scriptworker.version import __version_string__
-from taskcluster.aio import Queue
-from taskcluster.exceptions import TaskclusterFailure
 
 log = logging.getLogger(__name__)
 
@@ -365,7 +365,6 @@ def get_valid_worker_impls():
             validation functions.
 
     """
-    # TODO support taskcluster worker
     return immutabledict({"docker-worker": verify_docker_worker_task, "generic-worker": verify_generic_worker_task, "scriptworker": verify_scriptworker_task})
 
 
@@ -436,8 +435,8 @@ def check_interactive_docker_worker(link):
     Args:
         link (LinkOfTrust): the task link we're checking.
 
-    Returns:
-        list: the list of error errors.  Success is an empty list.
+    Raises:
+        CoTError: on interactive.
 
     """
     errors = []
@@ -449,7 +448,34 @@ def check_interactive_docker_worker(link):
             errors.append("{} is interactive: task.payload.env.TASKCLUSTER_INTERACTIVE!".format(link.name))
     except KeyError:
         errors.append("check_interactive_docker_worker: {} task definition is malformed!".format(link.name))
-    return errors
+    raise_on_errors(errors)
+
+
+# check_interactive_generic_worker {{{1
+def check_interactive_generic_worker(link):
+    """Given a task, make sure the task was not defined as interactive.
+
+    * ``task.payload.rdpInfo`` must be absent or False.
+    * ``task.payload.scopes`` must not contain a scope starting with ``generic-worker:allow-rdp:``
+
+    Args:
+        link (LinkOfTrust): the task link we're checking.
+
+    Raises:
+        CoTError: on interactive.
+
+    """
+    errors = []
+    log.info("Checking for {} {} interactive generic-worker".format(link.name, link.task_id))
+    try:
+        if link.task["payload"].get("rdpInfo"):
+            errors.append("{} is interactive: task.payload.rdpInfo!".format(link.name))
+        for scope in link.task["scopes"]:
+            if scope.startswith("generic-worker:allow-rdp:"):
+                errors.append("{} is interactive: {}!".format(link.name, scope))
+    except KeyError:
+        errors.append("check_interactive_generic_worker: {} task definition is malformed!".format(link.name))
+    raise_on_errors(errors)
 
 
 # verify_docker_image_sha {{{1
@@ -570,17 +596,14 @@ async def build_link(chain, task_name, task_id):
     """
     link = LinkOfTrust(chain.context, task_name, task_id)
     json_path = link.get_artifact_full_path("task.json")
-    try:
-        task_defn = await chain.context.queue.task(task_id)
-        link.task = task_defn
-        chain.links.append(link)
-        # write task json to disk
-        makedirs(os.path.dirname(json_path))
-        with open(json_path, "w") as fh:
-            fh.write(format_json(task_defn))
-        await build_task_dependencies(chain, task_defn, task_name, task_id)
-    except TaskclusterFailure as exc:
-        raise CoTError(str(exc))
+    task_defn = await retry_get_task_definition(chain.context.queue, task_id, exception=CoTError)
+    link.task = task_defn
+    chain.links.append(link)
+    # write task json to disk
+    makedirs(os.path.dirname(json_path))
+    with open(json_path, "w") as fh:
+        fh.write(format_json(task_defn))
+    await build_task_dependencies(chain, task_defn, task_name, task_id)
 
 
 async def build_task_dependencies(chain, task, name, my_task_id):
@@ -1136,12 +1159,21 @@ async def _get_additional_github_push_jsone_context(decision_link):
     github_repo = GitHubRepository(repo_owner, repo_name, context.config["github_oauth_token"])
     commit_data = await github_repo.get_commit(commit_hash)
 
-    committer_login = commit_data["committer"]["login"]
+    committer = commit_data["committer"] or {}
+    committer_login = committer.get("login")
     # https://github.com/mozilla-releng/scriptworker/issues/334: web-flow is the User used by
-    # GitHub to create some commits on github.com or the Github Desktop app. For sure, this user
-    # is not the one who triggered a push event. Let's fall back to the author login, instead.
-    if committer_login == "web-flow":
-        committer_login = commit_data["author"]["login"]
+    #   GitHub to create some commits on github.com or the Github Desktop app. For sure, this user
+    #   is not the one who triggered a push event. Let's fall back to the author login, instead.
+    # https://github.com/mozilla-releng/scriptworker/issues/477: the committer might not
+    #   be set for bots or improperly set up accounts. If everything else matches, let's not
+    #   get hung up on this.
+    if committer_login in ("web-flow", None):
+        author = commit_data.get("author") or {}
+        committer_login = author.get("login", commit_data.get("commit", {}).get("author", {}).get("login", "@@@unknown@@@"))
+    # This value could have been taken from `commit_data.parents[0]` too but
+    # it is more visible if picked up from `.taskcluster.yml` env vars
+    base_prefix = "{}_BASE_REV".format(context.config["source_env_prefix"])
+    before_hash = task["payload"]["env"].get(base_prefix)
 
     # Github users can have multiple emails. The commit_data contains
     # their primary email, but the task may contain a secondary email.
@@ -1154,8 +1186,10 @@ async def _get_additional_github_push_jsone_context(decision_link):
     #
     # [1] https://developer.github.com/v3/repos/commits/#get-a-single-commit
     # [2] https://developer.github.com/v3/activity/events/types/#pushevent
+
     return {
         "event": {
+            "before": before_hash,
             "after": commit_hash,
             "pusher": {"email": task_email},
             "ref": get_branch(task, source_env_prefix),
@@ -1216,7 +1250,7 @@ async def populate_jsone_context(chain, parent_link, decision_link, tasks_for):
         "taskId": None,
     }
 
-    if chain.context.config["cot_product"] in ("mobile", "mpd001", "application-services", "xpi"):
+    if chain.context.config["cot_product_type"] == "github":
         if tasks_for == "github-release":
             jsone_context.update(await _get_additional_github_releases_jsone_context(decision_link))
         elif tasks_for == "cron":
@@ -1228,8 +1262,8 @@ async def populate_jsone_context(chain, parent_link, decision_link, tasks_for):
         elif tasks_for == "github-push":
             jsone_context.update(await _get_additional_github_push_jsone_context(decision_link))
         else:
-            raise CoTError('Unknown tasks_for "{}" for cot_product "{}"!'.format(tasks_for, chain.context.config["cot_product"]))
-    else:
+            raise CoTError('Unknown tasks_for "{}" for github cot_product "{}"!'.format(tasks_for, chain.context.config["cot_product"]))
+    elif chain.context.config["cot_product_type"] == "hg":
         source_url = get_source_url(decision_link)
         project = await get_project(chain.context, source_url)
         jsone_context["repository"] = {
@@ -1245,7 +1279,11 @@ async def populate_jsone_context(chain, parent_link, decision_link, tasks_for):
         elif tasks_for == "cron":
             jsone_context.update(await _get_additional_hg_cron_jsone_context(parent_link, decision_link))
         else:
-            raise CoTError("Unknown tasks_for {}!".format(tasks_for))
+            raise CoTError('Unknown tasks_for "{}" for hg cot_product "{}"!'.format(tasks_for, chain.context.config["cot_product"]))
+    else:
+        raise CoTError(
+            'Unknown cot_product_type "{}" for cot_product "{}"!'.format(chain.context.config["cot_product_type"], chain.context.config["cot_product"])
+        )
 
     log.debug("{} json-e context:".format(parent_link.name))
     # format_json() breaks on lambda values; use pprint.pformat here.
@@ -1254,6 +1292,32 @@ async def populate_jsone_context(chain, parent_link, decision_link, tasks_for):
 
 
 # get_in_tree_template {{{1
+def build_taskcluster_yml_url(link):
+    """Build the url to the repo root ``.taskcluster.yml`` .
+
+    Args:
+        link (LinkOfTrust): the parent link to get the source url from.
+
+    Returns:
+        string: the ``.taskcluster.yml`` url
+
+    """
+    source_env_prefix = link.context.config["source_env_prefix"]
+    repo_url = get_repo(link.task, source_env_prefix)
+    repo_url = repo_url.replace("git@github.com:", "ssh://github.com/", 1)
+    revision = get_revision(link.task, source_env_prefix)
+    repo_parts = urlparse(repo_url)
+    if repo_parts.netloc == "github.com":
+        user, repo_name = extract_github_repo_owner_and_name(repo_url)
+        url = f"https://raw.githubusercontent.com/{user}/{repo_name}/{revision}/.taskcluster.yml"
+    elif repo_parts.netloc == "hg.mozilla.org":
+        url = f"{repo_parts.scheme}://{repo_parts.netloc}{repo_parts.path}/raw-file/{revision}/.taskcluster.yml"
+    else:
+        raise CoTError("Unsupported VCS server!")
+    log.debug(f"{link.name} .taskcluster.yml is at {url}")
+    return url
+
+
 async def get_in_tree_template(link):
     """Get the in-tree json-e template for a given link.
 
@@ -1271,21 +1335,16 @@ async def get_in_tree_template(link):
 
     """
     context = link.context
-    source_url = get_source_url(link)
-    if not source_url.endswith((".yml", ".yaml")):
-        raise CoTError("{} source url {} doesn't end in .yml or .yaml!".format(link.name, source_url))
+    source_url = build_taskcluster_yml_url(link)
+    repo_url = get_repo(link.task, link.context.config["source_env_prefix"])
 
     auth = None
-    if (any(vcs_rule.get("require_secret") for vcs_rule in context.config["trusted_vcs_rules"])) and "github.com" in source_url:
-        newurl = re.sub(
-            r"^(?:ssh://|https?://)(?:[^@/\:]*(?:\:[^@/\:]*)?@)?github.com(?:\:\d*)?/(?P<repopath>.*)/raw/(?P<sha>[a-zA-Z0-9]*)/(?P<filepath>.*)$",
-            r"https://raw.githubusercontent.com/\g<repopath>/\g<sha>/\g<filepath>",
-            source_url,
-        )
-        log.info("Converted source_url ({}) to new url ({})".format(source_url, newurl))
-        source_url = newurl
-        if context.config.get("github_oauth_token"):
-            auth = aiohttp.BasicAuth(context.config["github_oauth_token"])
+    if (
+        (repo_url.startswith(("ssh://", "git@github.com")) or any(vcs_rule.get("require_secret") for vcs_rule in context.config["trusted_vcs_rules"]))
+        and "github.com" in repo_url
+        and context.config.get("github_oauth_token")
+    ):
+        auth = aiohttp.BasicAuth(context.config["github_oauth_token"])
     tmpl = await load_json_or_yaml_from_url(context, source_url, os.path.join(context.config["work_dir"], "{}_taskcluster.yml".format(link.name)), auth=auth)
     return tmpl
 
@@ -1682,12 +1741,11 @@ async def verify_docker_worker_task(chain, link):
         CoTError: on failure.
 
     """
+    check_interactive_docker_worker(link)
     if chain != link:
-        # These two checks will die on `link.cot` if `link` is a ChainOfTrust
-        # object (e.g., the task we're running `verify_cot` against is a
-        # docker-worker task). So only run these tests if they are not the chain
-        # object.
-        check_interactive_docker_worker(link)
+        # This check will die on `link.cot` if `link` is a ChainOfTrust object
+        # (e.g., the task we're running `verify_cot` against is a docker-worker
+        # task). So only run this test if the link is not the chain object.
         verify_docker_image_sha(chain, link)
 
 
@@ -1703,7 +1761,9 @@ async def verify_generic_worker_task(chain, link):
         CoTError: on failure.
 
     """
-    pass
+    check_interactive_generic_worker(link)
+    # XXX once generic-worker supports docker, verify the docker image sha
+    #     if applicable.
 
 
 # verify_scriptworker_task {{{1
@@ -1927,7 +1987,7 @@ async def _async_verify_cot_cmdln(opts, tmp):
         context.config = dict(deepcopy(DEFAULT_CONFIG))
         context.credentials = read_worker_creds()
         context.queue = context.queue or Queue(session=session, options={"rootUrl": context.config["taskcluster_root_url"]})
-        context.task = await context.queue.task(opts.task_id)
+        context.task = await retry_get_task_definition(context.queue, opts.task_id, exception=CoTError)
         context.config.update(
             {
                 "cot_product": opts.cot_product,
@@ -1941,11 +2001,8 @@ async def _async_verify_cot_cmdln(opts, tmp):
         if os.environ.get("SCRIPTWORKER_GITHUB_OAUTH_TOKEN"):
             context.config["github_oauth_token"] = os.environ.get("SCRIPTWORKER_GITHUB_OAUTH_TOKEN")
         cot = ChainOfTrust(context, opts.task_type, task_id=opts.task_id)
-        await verify_chain_of_trust(cot, check_task=True)
-        log.info(format_json(cot.dependent_task_ids()))
-        log.info("{} : {}".format(cot.name, cot.task_id))
-        for link in cot.links:
-            log.info("{} : {}".format(link.name, link.task_id))
+        check_task = opts.no_check_task is False
+        await verify_chain_of_trust(cot, check_task=check_task)
 
 
 def verify_cot_cmdln(args=None, event_loop=None):
@@ -1980,11 +2037,14 @@ SCRIPTWORKER_GITHUB_OAUTH_TOKEN to an OAUTH token with read permissions to the r
     parser.add_argument("--cleanup", help="clean up the temp dir afterwards", dest="cleanup", action="store_true", default=False)
     parser.add_argument("--cot-product", help="the product type to test", default="firefox")
     parser.add_argument("--verify-sigs", help="enable signature verification", action="store_true", default=False)
+    parser.add_argument("--verbose", "-v", help="enable debug logging", action="store_true", default=False)
+    parser.add_argument("--no-check-task", help="skip verifying the taskId's cot status", action="store_true", default=False)
     opts = parser.parse_args(args)
     tmp = tempfile.mkdtemp()
     log = logging.getLogger("scriptworker")
-    log.setLevel(logging.DEBUG)
-    logging.basicConfig()
+    level = logging.DEBUG if opts.verbose else logging.INFO
+    log.setLevel(level)
+    logging.basicConfig(level=level)
     event_loop = event_loop or asyncio.get_event_loop()
     try:
         event_loop.run_until_complete(_async_verify_cot_cmdln(opts, tmp))
@@ -2003,7 +2063,7 @@ async def _async_create_test_workdir(task_id, path, queue=None):
         context.config = dict(deepcopy(DEFAULT_CONFIG))
         context.credentials = read_worker_creds()
         context.queue = queue or context.queue or Queue(session=session, options={"rootUrl": context.config["taskcluster_root_url"]})
-        context.task = await context.queue.task(task_id)
+        context.task = await retry_get_task_definition(context.queue, task_id, exception=CoTError)
         work_dir = os.path.abspath(path)
         context.config.update(
             {
